// XMLSerialization.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "XMLSerialization.h"
#include "XMLSerializer.h"
#include "Contact.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// The one and only application object

CWinApp theApp;
using namespace std;

void Serialize(LPCTSTR szXmlFile)
{
	CXMLSerializer ser(szXmlFile, _T("TestApp"), false);	 // IMP: true for deserialization

	CContact contact;
	contact.SetFirstName(_T("Meena"));
	contact.GetAddress()->SetCity(_T("Paris"));
	contact.GetEmailIds()->AddHead(_T("meena@hotmail.com"));
	contact.GetEmailIds()->AddHead(_T("meena@yahoo.com"));
	
	if(ser.Serialize(&contact))
	{
		_tprintf(_T("\nSerialized CContact class successfully to file %s\n"), szXmlFile);
	}
	else
	{
		_tprintf(_T("\nError serializing CContact class successfully to file %s\n"), szXmlFile);
	}
	
	/* we can serialize one more instance to the same xml file
	CContact contact2;
	contact2.SetFirstName(_T("Seema"));
	contact2.SetId(_T("002"));
	contact2.GetAddress()->SetCity(_T("London"));
	ser.Serialize(&contact2);
	*/
}

void Deserialize(LPCTSTR szXmlFile)
{
	CXMLSerializer ser(szXmlFile, _T("TestApp"), true);
										 // IMP: 'true' for deserialization
	
	CContact contact;
	CPtrList objects;
	int		 nObjects;
	
	nObjects = ser.Deserialize(&contact, objects);

	for(int n = 0; n < nObjects; n++)
	{
		CContact* obj = (CContact*)objects.GetAt(objects.FindIndex(n));
		_tprintf(obj->GetFirstName());
		_tprintf(_T("\n"));
		_tprintf(obj->GetAddress()->GetCity());
		_tprintf(_T("\n"));
		
		POSITION pos = obj->GetEmailIds()->GetHeadPosition();
		CString sEmail;
		while(pos)
		{
			sEmail = obj->GetEmailIds()->GetNext(pos);
			_tprintf(sEmail);
			_tprintf(_T("\n"));
		}

		pos = obj->GetAddressList()->GetHeadPosition();
		while(pos)
		{
			CAddress* address = (CAddress*)obj->GetAddressList()->GetNext(pos);
			_tprintf(address->GetCity());
			_tprintf(_T("\n"));
		}
		
		delete obj;
	}	
}

void Process(bool bSerialize, LPCTSTR szXmlFile)
{
	if(bSerialize)
	{
		Serialize(szXmlFile);
	}
	else
	{
		Deserialize(szXmlFile);
	}
}

void DisplaySyntax()
{
	_tprintf(_T("\nSyntax			: XMLSerialization [-S][-D] XMLFilePath"));
	_tprintf(_T("\nto serialize		: XMLSerialization -S c:\\contacts.xml"));
	_tprintf(_T("\nto de-serialize	: XMLSerialization -D c:\\contacts.xml\n"));
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int		nRetCode = 0;
	bool	bSerialize;
	
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{	
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		nRetCode = 1;
	}
	else
	{
		// e.g. XMLSerializer -S "c:\temp\contacts.xml" or
		//      XMLSerializer -S "c:\temp\contacts.xml" or
		if(argc != 3)
		{
			DisplaySyntax();
			return 0;
		}

		if(lstrcmpi(argv[1],  _T("-S")) == 0)
		{
			bSerialize = true;
		}
		else if(lstrcmpi(argv[1], _T("-D")) == 0)
		{
			bSerialize = false;
		}
		else
		{
			DisplaySyntax();
			return 0;
		}

		CoInitialize(NULL);
		Process(bSerialize, argv[2]);
		CoUninitialize();
	}

	return nRetCode;
}
